@@ -4,11 +4,12 @@ import Control.Applicative ((<$>))
 import Statistics.Sample.KernelDensity (kde)
 import Text.Hastache (MuType(..), defaultConfig, hastacheFile)
 import Text.Hastache.Context (mkStrContext)
-import qualified Data.Attoparsec as B
-import qualified Data.Attoparsec.Char8 as A
+import qualified Data.Attoparsec.ByteString as B
+import qualified Data.Attoparsec.ByteString.Char8 as A
 import qualified Data.ByteString as B
 import qualified Data.ByteString.Lazy as L
 import qualified Data.Vector.Unboxed as U
+import qualified Data.Text.Lazy.IO as TL
 
 csv = do
   B.takeTill A.isEndOfLine
@@ -20,4 +21,4 @@ main = do
   let xs = map (\(a,b) -> [a,b]) . U.toList . uncurry U.zip . kde 64 $ waits
       context "data" = MuVariable . show $ xs
   s <- hastacheFile defaultConfig "kde.tpl" (mkStrContext context)
-  L.writeFile "kde.html" s
+  TL.writeFile "kde.html" s
